#!/usr/bin/env python                                                     
# coding=utf-8

"""Transforms a DQBF formula to a QBF formula by bounding the paths under consideration."""

__author__ = "Leander Tentrup <tentrup@cs.uni-saarland.de>"
__copyright__ = "Copyright 2014, Saarland University"

import sys, os, threading

from lex import *
from parse import *
from dqbf2qbf_visitor import *
from qbf2qdimacs import QDIMACSfromQBF

THREADING=True

def printUsage():
    print('Usage: %s ["DQBF string" | -F file_name | stdin] [#paths]' % sys.argv[0])
    exit(1)
    
length = 0
dqpf_string = ''

def main():
    parse_tree = Parser(Lexer(Logics.DQBF, dqbf_string)).getAST()
    #print parse_tree
    
    visitor = DQBFVisitor(length)
    parse_tree.accept(visitor)
    qbf_tree = visitor.formula
    #print qbf_tree
    
    print(QDIMACSfromQBF(qbf_tree))

if __name__ == "__main__":
    
    threading.stack_size(2**21 * 256)
    sys.setrecursionlimit(10000000)
    
    process_stdin = False
    if not sys.stdin.isatty():
        process_stdin = True
        dqbf_string = sys.stdin.read()
        
        if len(sys.argv) == 2:
            length = sys.argv[1]
        else:
            printUsage()
    
    elif len(sys.argv) == 3:
        dqbf_string = sys.argv[1]
        length = sys.argv[2]
    elif len(sys.argv) == 4 and sys.argv[1] == '-F':
        file_handle = open(sys.argv[2], 'r')
        dqbf_string = file_handle.read()
        length = sys.argv[3]
    else:
        printUsage()
    
    if not length.isdigit():
        print('Illegal argument for length, should be a number')
        printUsage()
    
    length = int(length)
    
    if THREADING:
        thread = threading.Thread(target=main)
        thread.start()
    else:
         main()
