# coding=utf-8

"""Transforms a QBF parse tree to QDIMACS file format."""

__author__ = "Leander Tentrup <tentrup@cs.uni-saarland.de>"
__copyright__ = "Copyright 2014, Saarland University"

import os, tempfile, subprocess
from lex import *

CHECK_FOR_FREE_VARS=True

def execBool2CNF(bool_formula):
    bool_file = tempfile.NamedTemporaryFile()
    bool_file.write(str(bool_formula))
    
    bool_file.flush()
    
    output_file = tempfile.NamedTemporaryFile()

    proc = subprocess.Popen("%(path)s/bool2cnf -s %(file)s" % {'path': os.path.dirname(os.path.realpath(__file__)), 'file': bool_file.name}, stdout=output_file, shell=True)
    proc.communicate()
    
    output_file.seek(0)
    
    dimacs = ''
    
    max_num = 0
    ident_lookup = {}
    preamble = ''
    for line in output_file:
        line = line.strip()
        if line.startswith('c'):
            _,_,var,enc = line.split(' ')
            ident_lookup[var] = enc
        elif line.startswith('p'):
            max_num = int(line.split(' ')[2])
            preamble = line + '\n'
        else:
            dimacs += line + '\n'
    
    return dimacs, preamble, max_num, ident_lookup

def QDIMACSfromQBF(qbf_formula):
    
    assert qbf_formula.isQuantifier()
    
    child = qbf_formula
    while child.isQuantifier():
        child = child.operands[0]
    
    body = child
    
    dimacs, preamble, max_num, ident_lookup = execBool2CNF(str(body))
    assert max_num > 0
    
    header = ''
    quantification = []
    child = qbf_formula
    cnf_variables = set(range(1, max_num + 1))
    while child.isQuantifier():
        qmapping = {TokenType.A : 'a', TokenType.E: 'e'}
        line = [qmapping[child.qtype]]
        for var in child.variables:
            if var in ident_lookup:
                line.append(ident_lookup[var])
                quantification.append(var)
                cnf_variables.remove(int(ident_lookup[var]))
        line.append('0')
        
        header += ' '.join(line) + '\n'
        
        child = child.operands[0]
        
    if CHECK_FOR_FREE_VARS:
        for var in ident_lookup.keys():
            if var not in quantification:
                print('Free variable: %s' % var)
    
    cnf_introduction = 'e ' + ' '.join([str(i) for i in cnf_variables]) + ' 0'
        
    header += cnf_introduction + '\n'
    
    return preamble + header + dimacs